#include <stdlib.h>

#include "vtkRenderer.h"
#include "vtkRenderWindow.h"
#include "vtkActor.h"
#include "vtkActorCollection.h"
#include "vtkPolyDataMapper.h"
#include "vtkSphereSource.h"

#include "defs.h"
#include "types.h"
#include "protos.h"

extern int num_elements;

extern vtkPolyDataMapper **sphereMappers;
extern vtkPolyDataMapper *bondMappers[2];
extern vtkPolyDataMapper *lineMapper;
extern vtkPolyDataMapper *axesMapper;
extern vtkSphereSource   **spheres[3];

// Change the sphere objects to the specified complexity.
void change_sphere_complexity(vtkRenderWindow *renWindow, int complexity){
  

  for(int i = 0; i < num_elements; i++){
    sphereMappers[i]->SetInput(spheres[complexity][i]->GetOutput());
    sphereMappers[i]->Update();
  }

}

// Change the bond objects to the specified complexity.
void change_bond_complexity(vtkRenderer *ren, vtkRenderWindow *renWindow, int complexity){
  vtkActor *current;
  vtkActorCollection *actors = ren->GetActors();
  int numItems, i;

  numItems = actors->GetNumberOfItems();

  actors->InitTraversal();
  current = actors->GetNextActor();

  if(complexity == LINE_BOND){
    for(i = 0; i < numItems; i++){
      if(current->GetMapper() == bondMappers[0] ||
	 current->GetMapper() == bondMappers[1]){
	current->SetMapper(lineMapper);
      }
      current = actors->GetNextActor();
    }
  }
  else if(complexity == STRC_BOND){
    for(i = 0; i < numItems; i++){
      if(current->GetMapper() == lineMapper ||
	 current->GetMapper() == bondMappers[1]){
	current->SetMapper(bondMappers[0]);
      }
      current = actors->GetNextActor();
    }
  }
  else if(complexity == TAPC_BOND){
    for(i = 0; i < numItems; i++){
      if(current->GetMapper() == lineMapper ||
	 current->GetMapper() == bondMappers[0]){
	current->SetMapper(bondMappers[1]);
      }
      current = actors->GetNextActor();
    }
  }

}
