#include <stdio.h>
#include <string.h>

#include "defs.h"
#include "types.h"
#include "protos.h"

/*
 * This routine defines the material properties, and sets
 * the flag colors. Also computes the average atom radius.
 *
 * Input: list of elements, flags
 * Output: changed flags, graphics
 */

/* auto-pack colors */
#define COLOR_PACK(x) x.pack = ( \
   (long) (255*x.red  ) * 0x00000001 | \
   (long) (255*x.green) * 0x00000100 | \
   (long) (255*x.blue ) * 0x00010000 )

/* invert color for labels */
#define INV_COLOR(x) (x>0.5 ? x-0.5 : x+0.5)

void color_setup( ELEMENT_LIST *head, DRAW_FLAGS *flags ){
  float avg_color, avg_radius = 0.0;
  int num_radii = 0;

  ELEMENT_LIST *current;
  int sequence = 1;

  for (current=head; current != NULL; current=current->next) {

    /* Add to average radius tally */
    avg_radius += current->radius;
    num_radii++;

    /* set up the label color for this atom */
    /* idea is to find the color intensity level, then pick a */
    /* gray value that will stand out in comparison           */
    
    avg_color = 0.2989 * current->color.red 
               + 0.5866 * current->color.green
               + 0.1144 * current->color.blue;
    avg_color = 0.25 + 0.75 * avg_color;
    current->label_color.red   = INV_COLOR(avg_color);
    current->label_color.green = INV_COLOR(avg_color);
    current->label_color.blue  = INV_COLOR(avg_color);
    
    /* check for background, axis, bond, or frame color defn */
    if ( strcmp(BACKG_LABEL, current->label) == 0 ) {
      flags->back_color.red   = current->color.red;
      flags->back_color.green = current->color.green;
      flags->back_color.blue  = current->color.blue;
    }
    else if ( strcmp(BONDC_LABEL, current->label) == 0 ) {
      flags->bond_color.red   = current->color.red;
      flags->bond_color.green = current->color.green;
      flags->bond_color.blue  = current->color.blue;
    }
    else if ( strcmp(AXISC_LABEL, current->label) == 0 ) {
      flags->axis_color.red   = current->color.red;
      flags->axis_color.green = current->color.green;
      flags->axis_color.blue  = current->color.blue;
#ifdef MOVIE
    }
    else if ( strcmp(FRMCT_LABEL, current->label) == 0 ) {
      flags->cntr_color.red   = current->color.red;
      flags->cntr_color.green = current->color.green;
      flags->cntr_color.blue  = current->color.blue;
#endif
    }
    
  } // list through elements

  // Compute and store average radius
  flags->avg_radius = avg_radius*0.40/num_radii;



}
