/*
 * Rotmole / Rotmovie source code
 *
 * $RCSfile: create_bond_table.c,v $
 *
 * $Log: create_bond_table.c,v $
 * Revision 3.0  1994/11/21 14:31:21  brabec
 * *** empty log message ***
 *
 */

#include <stdio.h>

#include "defs.h"
#include "types.h"
#include "protos.h"

/*
 * This routine generates a bond table for the given frame.
 * It will not run unless the table is empty;
 *
 * Input: frame with no bond table
 * Output: frame with bond table
 */

void create_bond_table( FRAME_LIST *frame )
{

   ATOM_LIST *at1, *at2;

   BOND_LIST *cur = NULL;

   float dx,dy,dz,rad;
   float bond_dist;


   if (frame->bonds != NULL) return;

/* N^2 loop over all possible pairs */
   for (at1 = frame->atoms; at1 != NULL; at1 = at1->next) 
   if (at1->info->can_bond) {

     for (at2 = at1->next; at2 != NULL; at2 = at2->next) 
     if (at2->info->can_bond) {

       bond_dist = at2->info->radius + at1->info->radius;
/*
printf("1=%s 2=%s %f %f\n",at1->info->label,at2->info->label,at1->info->radius,at2->info->radius);
printf("1=%s 2=%s %f %f\n",at1->info->label,at2->info->label,bond_dist*bond_dist,rad);
*/

       dx = ABS(at2->x - at1->x);
       if (dx < bond_dist) {
         dy = ABS(at2->y - at1->y);
         if (dy < bond_dist) {
           dz = ABS(at2->z - at1->z);
           if (dz < bond_dist) {
             rad = dx*dx + dy*dy + dz*dz;
             if (rad < bond_dist*bond_dist) {


               if (cur == NULL) {
                 cur = (BOND_LIST *) acquire( sizeof(BOND_LIST) );
                 Check_ptr(cur);
                 frame->bonds = cur;
               } else {
                 cur->next = (BOND_LIST *) acquire( sizeof(BOND_LIST) );
                 Check_ptr(cur->next);
                 cur=cur->next;
               }

               cur->next = NULL;
               cur->end1 = at1;
               cur->end2 = at2;

             } /* if rad */
           } /* if dz */
         } /* if dy */
       } /* if dx */

     }

   }

}


/* end of create_bond_table.cxx */
