#include <stdio.h>

#include "vtkLineSource.h"
#include "vtkCylinderSource.h"
#include "vtkPolyDataMapper.h"

#include "defs.h"
#include "types.h"
#include "protos.h"

extern vtkPolyDataMapper *lineMapper;
extern vtkPolyDataMapper *bondMappers[2];

// Create the mapper objects for the molecular bonds.
void create_bonds(DRAW_FLAGS *flags){
  vtkCylinderSource *bonds[2];

  // Create 3 different complexity bonds
  vtkLineSource *line = vtkLineSource::New();
  line->SetPoint1(0.0, 0.0, 0.0);
  line->SetPoint2(0.0, 0.0, 1.0);

  bonds[0] = vtkCylinderSource::New();
  bonds[0]->SetRadius(flags->bond_size);
  bonds[0]->SetResolution(6);
  bonds[0]->SetHeight(1.0);

  bonds[1] = vtkCylinderSource::New();
  bonds[1]->SetRadius(flags->bond_size);
  bonds[1]->SetResolution(10);
  bonds[1]->SetHeight(1.0);
  

  // Create a mapper for each bond
  lineMapper = vtkPolyDataMapper::New();
  lineMapper->SetInput(line->GetOutput());

  bondMappers[0] = vtkPolyDataMapper::New();
  bondMappers[0]->SetInput(bonds[0]->GetOutput());

  bondMappers[1] = vtkPolyDataMapper::New();
  bondMappers[1]->SetInput(bonds[1]->GetOutput());

  
  // Delete the source objects
  line->Delete();
  bonds[0]->Delete();
  bonds[1]->Delete();
}
