#include "vtkSphereSource.h"
#include "vtkPolyDataMapper.h"

#include "defs.h"
#include "types.h"
#include "protos.h"

extern int num_elements;
extern vtkSphereSource **spheres[3];
extern vtkPolyDataMapper **sphereMappers;

void create_spheres(DRAW_FLAGS *flags, ELEMENT_LIST *element){
  int complexity;
  float radius;

  // Get the sphere radius and complexity
  radius = flags->sphr_size;
  complexity = flags->sphr_complex;

  
  // Create 3 sphere sources for each element (one for each complexity) 
  spheres[0] = (vtkSphereSource **) malloc(4 * num_elements);
  spheres[1] = (vtkSphereSource **) malloc(4 * num_elements);
  spheres[2] = (vtkSphereSource **) malloc(4 * num_elements);
  for(int i = 0; i < num_elements; ){

    // Skip the default elements that store basic color info
    if( strncmp(element->label, BACKG_LABEL, LABEL_LEN) != 0 &&
	strncmp(element->label, BONDC_LABEL, LABEL_LEN) != 0 &&
	strncmp(element->label, AXISC_LABEL, LABEL_LEN) != 0 &&
	strncmp(element->label, FRMCT_LABEL, LABEL_LEN) != 0 ){

      spheres[0][i] = vtkSphereSource::New();
      spheres[0][i]->SetPhiResolution(6);
      spheres[0][i]->SetThetaResolution(6);
      spheres[0][i]->SetRadius(element->radius * radius);

      spheres[1][i] = vtkSphereSource::New();
      spheres[1][i]->SetPhiResolution(8);
      spheres[1][i]->SetThetaResolution(8);
      spheres[1][i]->SetRadius(element->radius * radius);

      spheres[2][i] = vtkSphereSource::New();
      spheres[2][i]->SetPhiResolution(10);
      spheres[2][i]->SetThetaResolution(10);
      spheres[2][i]->SetRadius(element->radius * radius);
      
      // This is used later to easily index the array of sphere mappers.
      // It serves as the link between an element and the appropriate mapper.
      element->sphere_index = i;
      i++;
    }
    element = element->next;
  }

 
  // Likewise, create a mapper for each of our sphere sources
  // and set its source to the current sphere complexity.
  sphereMappers = (vtkPolyDataMapper **) malloc(4 * num_elements);
  for(int i = 0; i < num_elements; i++){
    sphereMappers[i] = vtkPolyDataMapper::New();
    sphereMappers[i]->SetInput(spheres[complexity][i]->GetOutput());
  }

}
