#include <stdlib.h>

#include "vtkRenderer.h"
#include "vtkRenderWindow.h"
#include "vtkActor.h"
#include "vtkActorCollection.h"
#include "vtkPolyDataMapper.h"

#include "defs.h"
#include "types.h"
#include "protos.h"

extern int num_elements;

extern vtkPolyDataMapper **sphereMappers;
extern vtkPolyDataMapper *bondMappers[2];
extern vtkPolyDataMapper *lineMapper;
extern vtkPolyDataMapper *axesMapper;
extern vtkPolyDataMapper **labelMappers;
extern vtkPolyDataMapper *surfaceMapper;

void delete_bonds(vtkRenderer *ren){
  vtkActor *current;
  vtkActorCollection *actors = ren->GetActors();
  int numItems;

  numItems = actors->GetNumberOfItems();

  actors->InitTraversal();
  current = actors->GetNextActor();
  for(int i = 0; i < numItems; i++){
    if(current->GetMapper() == bondMappers[0] ||
       current->GetMapper() == bondMappers[1] ||
       current->GetMapper() == lineMapper){
      ren->RemoveActor(current);
    }
    current = actors->GetNextActor();
  }

}

void delete_atoms(vtkRenderer *ren){
  vtkActor *current;
  vtkActorCollection *actors = ren->GetActors();
  int numItems, isAtom;
  int i, j;
  
  numItems = actors->GetNumberOfItems();

  actors->InitTraversal();
  current = actors->GetNextActor();
  for(i = 0; i < numItems; i++){
    isAtom = 0;
    j = 0;
    while(!isAtom && j < num_elements){
      if(current->GetMapper() == sphereMappers[j]) isAtom = 1;
      j++;
    }
    if(isAtom) ren->RemoveActor(current);
    current = actors->GetNextActor();
  }

}

void delete_axes(vtkRenderer *ren){
  vtkActor *current;
  vtkActorCollection *actors = ren->GetActors();
  int numItems;

  numItems = actors->GetNumberOfItems();

  actors->InitTraversal();
  current = actors->GetNextActor();
  for(int i = 0; i < numItems; i++){
    if(current->GetMapper() == axesMapper){
      ren->RemoveActor(current);
    }
    current = actors->GetNextActor();
  }


}

void delete_atom_labels(vtkRenderer *ren){
  vtkActor *current;
  vtkActorCollection *actors = ren->GetActors();
  int numItems;
  int i, j;
  int isLabel;

  numItems = actors->GetNumberOfItems();

  actors->InitTraversal();
  current = actors->GetNextActor();
  for(i = 0; i < numItems; i++){
    isLabel = 0;
    j = 0;
    while(!isLabel && j < num_elements){
      if(current->GetMapper() == labelMappers[j]) isLabel = 1;
      j++;
    }
    if(isLabel) ren->RemoveActor(current);
    current = actors->GetNextActor();
  }


}

void delete_surfaces(vtkRenderer *ren){
  vtkActor *current;
  vtkActorCollection *actors = ren->GetActors();
  int numItems;

  numItems = actors->GetNumberOfItems();

  actors->InitTraversal();
  current = actors->GetNextActor();
  for(int i = 0; i < numItems; i++){
    if(current->GetMapper() == surfaceMapper){
      ren->RemoveActor(current);
    }
    current = actors->GetNextActor();
  }

}

void delete_frame(vtkRenderer *ren){
  vtkActor *current;
  vtkActorCollection *actors = ren->GetActors();
  int numItems;

  numItems = actors->GetNumberOfItems();

  actors->InitTraversal();
  current = actors->GetNextActor();
  for(int i = 0; i < numItems; i++){
    ren->RemoveActor(current);
    current = actors->GetNextActor();
  }
  
}

