#include <stdlib.h>

#include "vtkVolume16Reader.h"
#include "vtkPolyDataMapper.h"
#include "vtkStripper.h"
#include "vtkContourFilter.h"
#include "vtkMarchingCubes.h"
#include "vtkStructuredPointsReader.h"
#include "vtkLookupTable.h"

#include "defs.h"
#include "types.h"
#include "protos.h"

//extern vtkContourFilter *isoExtractor;
extern vtkMarchingCubes *isoExtractor;
extern vtkPolyDataMapper *surfaceMapper;
 
extern SURFACE_INFO surface_info;

// Open the electron density file(file must be a vtk format data file).
// Also, create the contour filter.
void open_density_file(char *filename){
  float scalarRange[2];

  if(isoExtractor != NULL){
    isoExtractor->Delete();
    isoExtractor = NULL;
  }

  // Read the density data from a vtk file
  vtkStructuredPointsReader *reader = vtkStructuredPointsReader::New();
  reader->SetFileName(filename);
  reader->Update();
  reader->GetOutput()->GetScalarRange(scalarRange);
  
  // Find the min/max density values in the data set
  surface_info.min_density = scalarRange[0];
  surface_info.max_density = scalarRange[1];

  // Create the contour filter.
  // vtkMarchingCubes creates a higher quality image and is faster than
  // vtkContourFilter.  
  //isoExtractor = vtkContourFilter::New();
  isoExtractor = vtkMarchingCubes::New();
  isoExtractor->SetInput(reader->GetOutput());

  reader->Delete();

  // Initialize surface_info structure data
  surface_info.opacity = 0.5;
  surface_info.specular = 0.1;
  surface_info.spec_power = 20;

  
}


void init_contour_filter(){ 
  int i;

  // Create a color map.
  // This color map transitions from red to blue and has a constant alpha value.
  vtkLookupTable *colormap = vtkLookupTable::New();
  colormap->SetHueRange(0.6667, 0.0);
  colormap->SetAlphaRange(surface_info.opacity, surface_info.opacity);
  colormap->Build();

  // Set the values that the isoExtractor will create isosurfaces around.
  for(i = 0; i < surface_info.num_surfaces; i++){
    isoExtractor->SetValue(i, surface_info.contour_values[i]);
  }
  isoExtractor->SetNumberOfContours(surface_info.num_surfaces);
  
  // Strip the surfaces created by isoExtractor of unneccesary traingles to
  // make the image a little prettier and faster to render.
  vtkStripper *isoStripper = vtkStripper::New();
  isoStripper->SetInput(isoExtractor->GetOutput());

  // If a surface mapper already exists, delete it
  if(surfaceMapper != NULL) surfaceMapper->Delete();
  
  // Create the surface mapper, set its input, and set its color lookup table
  surfaceMapper = vtkPolyDataMapper::New();
  surfaceMapper->SetInput(isoStripper->GetOutput());
  surfaceMapper->SetLookupTable(colormap);
  
  // Map the min density to the first entry in the color table and the max density
  // to the last entry.
  surfaceMapper->SetScalarRange(surface_info.min_density, surface_info.max_density);

  colormap->Delete();
  isoStripper->Delete();
}
