#include <stdio.h>
#include <math.h>

#include "vtkTransform.h"

#include "defs.h"
#include "types.h"
#include "protos.h"

// Calculates the distance between 2 points
float distance(float x1, float y1, float z1, float x2, float y2, float z2){

  return sqrt( ((z2-z1)*(z2-z1)) + ((y2-y1)*(y2-y1)) + ((x2-x1)*(x2-x1)) );
}

// Calculates the angle between 2 vectors, with (x2, y2, z2) being the
// point shared by both vectors.
float angle(float x1, float y1, float z1, float x2, float y2, float z2,
	    float x3, float y3, float z3){
  double px, py, pz, qx, qy, qz;
  double magp, magq;
  double dotproduct;
  double radangle;

  px = x1 - x2;
  py = y1 - y2;
  pz = z1 - z2;

  qx = x3 - x2;
  qy = y3 - y2;
  qz = z3 - z2;

  dotproduct = (px * qx) + (py * qy) + (pz * qz);
  magp = sqrt( (px * px) + (py * py) + (pz * pz) );
  magq = sqrt( (qx * qx) + (qy * qy) + (qz * qz) );

  radangle = acos( dotproduct / (magp * magq) );

  return radangle * (180.0 / M_PI);
}

vtkTransform* getTransformation(float x1, float y1, float z1, float x2, float y2, float z2){
  vtkTransform *trans = vtkTransform::New();
  double xrot, yrot;
  double vx, vy, vz, magnitude;
  double px, py, pz;
  double rad2degrees = 180.0 / M_PI;

  // convert to a vector representation of the line
  vx = x2 - x1;
  vy = y2 - y1;
  vz = z2 - z1;

  // calculate the magnitude of the vector
  magnitude = sqrt(vx * vx + vy * vy + vz * vz);

  // normalize the vector
  vx = vx / magnitude;
  vy = vy / magnitude;
  vz = vz / magnitude;

  // Calculate the rotations depending on which octet the vector belongs.
  // I know this can be simplified, but hey, I've got a lot to do.
  if(vx == 0.0 && vy == 0.0 && vz >= 0.0){
    xrot = 0.0;
    yrot = 0.0;
  }
  else if(vx == 0.0 && vz == 0.0 && vy >= 0.0){
    xrot = -90.0;
    yrot = 0.0;
  }
  else if(vy == 0.0 && vz == 0.0 && vx >= 0.0){
    xrot = 0.0;
    yrot = 90.0;
  }
  else if(vx == 0.0 && vy == 0.0 && vz < 0.0){
    xrot = 0.0;
    yrot = 180.0;
  }
  else if(vx == 0.0 && vz == 0.0 && vy < 0.0){
    xrot = 90.0;
    yrot = 0.0;
  }
  else if(vy == 0.0 && vz == 0.0 && vx < 0.0){
    xrot = 0.0;
    yrot = -90.0;
  }
  else if(vx >= 0.0 && vy >= 0.0 && vz >= 0.0){
    xrot = atan(vy / (sqrt(vx*vx+vz*vz))) * rad2degrees * -1.0;
    yrot = atan(vx / vz) * rad2degrees;
  }
  else if(vx < 0.0 && vy >= 0.0 && vz >= 0.0){ 
    xrot = atan(vy / (sqrt(vx*vx+vz*vz))) * rad2degrees * -1.0;
    yrot = atan(vx / vz) * rad2degrees;
  }
  else if(vx >= 0.0 && vy < 0.0 && vz >= 0.0){ 
    xrot = atan(vy / (sqrt(vx*vx+vz*vz))) * rad2degrees * -1.0;
    yrot = atan(vx / vz) * rad2degrees;
  }
  else if(vx < 0.0 && vy < 0.0 && vz >= 0.0){ 
    xrot = atan(vy / (sqrt(vx*vx+vz*vz))) * rad2degrees * -1.0;
    yrot = atan(vx / vz) * rad2degrees;
  }
  else if(vx >= 0.0 && vy >= 0.0 && vz < 0.0){ 
    xrot = atan(vy / (sqrt(vx*vx+vz*vz))) * rad2degrees * -1.0;
    yrot = (atan(vx / vz) * rad2degrees * -1.0);
    yrot = yrot + 2.0 * (90.0 - fabs(yrot));
  }
  else if(vx < 0.0 && vy >= 0.0 && vz < 0.0){ 
    xrot = atan(vy / (sqrt(vx*vx+vz*vz))) * rad2degrees * -1.0;
    yrot = (atan(vx / vz) * rad2degrees * -1.0);
    yrot = yrot - 2.0 * (90.0 - fabs(yrot));
   }
  else if(vx >= 0.0 && vy < 0.0 && vz < 0.0){ 
    xrot = atan(vy / (sqrt(vx*vx+vz*vz))) * rad2degrees * -1.0;
    yrot = (atan(vx / vz) * rad2degrees * -1.0);
    yrot = yrot + 2 * (90.0 - fabs(yrot));
  }
  else if(vx < 0.0 && vy < 0.0 && vz < 0.0){ 
    xrot = atan(vy / (sqrt(vx*vx+vz*vz))) * rad2degrees * -1.0;
    yrot = (atan(vx / vz) * rad2degrees * -1.0);
    yrot = yrot - 2.0 * (90.0 - fabs(yrot));
  }

  // Now we assemble our transformation matrix
  trans->Translate(x1, y1, z1);
  trans->RotateY(yrot);
  trans->RotateX(xrot);
  trans->Scale(1.0, 1.0, magnitude);

  return trans;
  
}
