#include <stdio.h>
#include <gtk/gtk.h>

#include "vtkRenderWindowInteractor.h"
#include "vtkXRenderWindowInteractor.h"
#include "vtkPNMWriter.h"
#include "vtkBMPWriter.h"
#include "vtkPostScriptWriter.h"
#include "vtkWindowToImageFilter.h"
#include "vtkContourFilter.h"
#include "vtkMarchingCubes.h"

#include "defs.h"
#include "types.h"
#include "protos.h"
#include "gtk_protos.h"

extern vtkRenderer *ren;
extern vtkRenderWindow *renWindow;
extern vtkXRenderWindowInteractor *iren;
extern vtkMarchingCubes *isoExtractor;
//extern vtkContourFilter *isoExtractor;

extern DRAW_FLAGS draw_flags;
extern SURFACE_INFO surface_info;
extern ELEMENT_LIST *element_head;
extern FRAME_LIST *frame_head;
extern FRAME_LIST *frame_curr;
extern FRAME_LIST *frame_tail;
extern ATOM_LIST *pickedAtoms[3];

GtkWidget *control_window;       // The main GTK window
GtkWidget *frame_counter;        // Label widget to show the current frame
GtkWidget *numSurfacesCombo;     // Choose the number of isosurfaces
GtkWidget *minEntry, *maxEntry;  // Min/Max density entry widgets
GtkWidget *opacityEntry;         // Isosurface opacity entry widget
GtkWidget *cvEntry[6];           // Contour value entry widgets

const char pixmapdir[] = "/usr/share/rotmovie/";

void create_control_window(){
  GtkWidget *menubar, *hbox, *vbox, *table, *vbox2;

  /* Create new top level window */
  control_window = gtk_window_new( GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(control_window), "Rotmovie");
  gtk_container_border_width(GTK_CONTAINER(control_window), 2);
  
  /* Quit from main if got delete event */
  gtk_signal_connect(GTK_OBJECT(control_window), "delete_event", 
		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);

  vbox = gtk_vbox_new(FALSE, 0);

  // Create the menu
  create_menu(control_window, &menubar);
  gtk_box_pack_start(GTK_BOX(vbox), menubar, TRUE, FALSE, 0);
  gtk_widget_show(menubar);
  gtk_widget_show(GTK_WIDGET(control_window));

  // Create the movie controls
  table = create_player_buttons();
  gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, FALSE, 0);
  gtk_widget_show(table);

  // Create the isosurface extraction controls
  vbox2 = gtk_vbox_new(FALSE, 0);
  populateVbox2(vbox2);
  gtk_box_pack_start(GTK_BOX(vbox), vbox2, TRUE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(vbox2));

  gtk_container_add(GTK_CONTAINER(control_window), GTK_WIDGET(vbox));
  gtk_widget_show(GTK_WIDGET(vbox));
  //gtk_widget_show(GTK_WIDGET(control_window));
}

void create_menu(GtkWidget *window, GtkWidget **menubar){
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  static GtkItemFactoryEntry menu_items[] = {
    { "/_File",                     NULL,         NULL, 0, "<Branch>" },
    { "/File/_Open Density File",   "<control>O", create_density_file_window, 0, NULL },
    { "/File/Save Coordinates",     NULL,         create_saveCoord_file_window, 0, NULL},
    { "/File/Save PPM",             NULL,         create_writePPM_file_window, 0, NULL},
    { "/File/Save BMP",             NULL,         create_writeBMP_file_window, 0, NULL},
    { "/File/Save Postscript",      NULL,         create_writePS_file_window, 0, NULL},
    { "/File/sep1",                 NULL,         NULL, 0, "<Separator>" },
    { "/File/_Quit",                "<control>Q", gtk_main_quit, 0, NULL },
    { "/_Options",                  NULL,         NULL, 0, "<Branch>" },
    { "/Options/Sphere Complexity", NULL,         NULL, 0, "<LastBranch>" },
    { "/Options/Sphere Complexity/Normal", NULL,  set_sph_cmplx0, 0, "<RadioItem>" },
    { "/Options/Sphere Complexity/Good", NULL,    set_sph_cmplx1, 0, "<RadioItem>" },
    { "/Options/Sphere Complexity/Best", NULL,    set_sph_cmplx2, 0, "<RadioItem>" },
    { "/Options/Bond Complexity",   NULL,         NULL, 0, "<LastBranch>" },
    { "/Options/Bond Complexity/Line", NULL,      set_bond_cmplx0, 0, "<RadioItem>" },
    { "/Options/Bond Complexity/Cylinder", NULL,  set_bond_cmplx1, 0, "<RadioItem>" },
    { "/Options/Bond Complexity/Cylinder 2", NULL,set_bond_cmplx2, 0, "<RadioItem>" },
    { "/Options/Hide Axes",         NULL,         toggle_axes, 0, "<ToggleItem>" },
    { "/Options/Hide Bonds",        NULL,         toggle_bonds, 0, "<ToggleItem>" },
    { "/Options/Hide Atoms",        NULL,         toggle_atoms, 0, "<ToggleItem>" },
    { "/Options/Draw Atom Labels",  NULL,         toggle_alabels, 0, "<ToggleItem>" },
    { "/Options/Orthographic Proj", NULL,         toggle_projection, 0, "<ToggleItem>"},
    { "/Vtk",                       NULL,         NULL, 0, "<Branch>" },
    { "/Vtk/Start _Interactor",     "<control>I", iren_start,  0, NULL},
    { "/_Help",                   NULL,         NULL, 0, "<LastBranch>" },
    { "/_Help/About",             NULL,         NULL, 0, NULL },
  };
  gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
  
  
  accel_group = gtk_accel_group_new();
  
  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);
  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);
  gtk_accel_group_attach (accel_group, GTK_OBJECT (window));
  *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}

void populateVbox2(GtkWidget *vbox2){
  GtkWidget *hbox, *vbox, *label, *button;
  GList *numItemList = NULL;
  int i;

  numItemList = g_list_append(numItemList, (void*) "1");
  numItemList = g_list_append(numItemList, (void*) "2");
  numItemList = g_list_append(numItemList, (void*) "3");
  numItemList = g_list_append(numItemList, (void*) "4"); 
  numItemList = g_list_append(numItemList, (void*) "5");
  numItemList = g_list_append(numItemList, (void*) "6");

  vbox = gtk_vbox_new(FALSE, 5);
  
  // Min and Max Density Entry widgets
  hbox = gtk_hbox_new(FALSE, 5);

  label = gtk_label_new("Min Density");
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(label));

  minEntry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(minEntry), 12);
  gtk_widget_set_usize(minEntry, 90, 22);  
  gtk_box_pack_start(GTK_BOX(hbox), minEntry, FALSE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(minEntry));
    
  label = gtk_label_new("Max Density");
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(label));

  maxEntry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(maxEntry), 12);
  gtk_widget_set_usize(maxEntry, 90, 22);  
  gtk_box_pack_start(GTK_BOX(hbox), maxEntry, FALSE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(maxEntry));

  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(hbox));

  // Isosurface opacity entry widget
  hbox = gtk_hbox_new(FALSE, 5);
  label = gtk_label_new("Isosurface Opacity");
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(label));

  opacityEntry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(opacityEntry), 5);
  gtk_widget_set_usize(opacityEntry, 50, 22);  
  gtk_box_pack_start(GTK_BOX(hbox), opacityEntry, FALSE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(opacityEntry));

  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(hbox));

  // Number of isosurfaces combo box
  hbox = gtk_hbox_new(FALSE, 5);
  label = gtk_label_new("Number of Density Isosurfaces");
  numSurfacesCombo = gtk_combo_new();
  gtk_combo_set_popdown_strings(GTK_COMBO(numSurfacesCombo), numItemList);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(numSurfacesCombo)->entry), "changed",
		     GTK_SIGNAL_FUNC(setNumSensitivity), NULL);

  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), numSurfacesCombo, FALSE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(label));
  gtk_widget_show(GTK_WIDGET(numSurfacesCombo));

  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(hbox));

  // Isosurface contour value entry widgets
  for(i = 0; i < 6; i++){
    hbox = gtk_hbox_new(FALSE, 5);

    label = gtk_label_new("Contour Value");
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    gtk_widget_show(GTK_WIDGET(label));

    cvEntry[i] = gtk_entry_new();
    gtk_entry_set_max_length(GTK_ENTRY(cvEntry[i]), 12);
    gtk_widget_set_usize(cvEntry[i], 90, 22);

    gtk_box_pack_start(GTK_BOX(hbox), cvEntry[i], FALSE, FALSE, 0);
    gtk_widget_show(GTK_WIDGET(cvEntry[i]));
    
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show(GTK_WIDGET(hbox));
  }

  // Render isosurface button
  hbox = gtk_hbox_new(FALSE, 5);
  button = gtk_button_new_with_label("Render");
  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(render_isosurfaces),
		     NULL);
  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(button));

  // Hide isosurface button
  button = gtk_button_new_with_label("Hide");
  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(hide_isosurfaces),
		     NULL);
  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(button));

  // Pack the hide and render buttons
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show(GTK_WIDGET(hbox));

  
  gtk_box_pack_start(GTK_BOX(vbox2), vbox, FALSE, FALSE, 0);
  gtk_widget_show(vbox);

  // Initialize the states of the cvEntry widgets and set the
  // number of surfaces to one.
  setNumSensitivity();

}

// Changes the state of the contour value entry widgets to active/inactive
// depending on the number of contours chosen.  Also, sets the num_surfaces
// field in the SURFACE_INFO structure.
void setNumSensitivity(){
  int i;
  
  surface_info.num_surfaces = atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(numSurfacesCombo)->entry)));
  
  for(i = 0; i < surface_info.num_surfaces; i++){
    gtk_widget_set_sensitive(GTK_WIDGET(cvEntry[i]), 1);
  }

  for(i = 5; i >= surface_info.num_surfaces; i--){
    gtk_widget_set_sensitive(GTK_WIDGET(cvEntry[i]), 0);
  }
}

// Set the min and max density values in the SURFACE_INFO structure
void set_minmax_values(){

  surface_info.min_density = atof((char*) gtk_editable_get_chars(GTK_EDITABLE(minEntry), 0, -1));
  surface_info.max_density = atof((char*) gtk_editable_get_chars(GTK_EDITABLE(maxEntry), 0, -1));

}

// Set the contour values in the SURFACE_INFO structure
void set_contour_values(){
  int i;

  for(i = 0; i < surface_info.num_surfaces; i++){
    surface_info.contour_values[i] = atof((char*) gtk_editable_get_chars(GTK_EDITABLE(cvEntry[i]), 0, -1));
  }


}

// Set the opacity of the isosurfaces
void set_opacity_value(){

  surface_info.opacity = atof((char*) gtk_editable_get_chars(GTK_EDITABLE(opacityEntry), 0, -1));

}

// Start the vtk window interactor
void iren_start(){

  iren->Start();

}

// Set the sphere(atom) complexity to simple
void set_sph_cmplx0(){
  if(draw_flags.sphr_complex != NORM_SPH){
    change_sphere_complexity(renWindow, 0);
    draw_flags.sphr_complex = 0;
  }

  // Update the render window to reflect the changes
  renWindow->Render();
}

// Set the sphere(atom) complexity to medium
void set_sph_cmplx1(){
  if(draw_flags.sphr_complex != GOOD_SPH){
    change_sphere_complexity(renWindow, 1);
    draw_flags.sphr_complex = 1;
  }

  // Update the render window to reflect the changes
  renWindow->Render();
}

// Set the sphere(atom) complexity to complex
void set_sph_cmplx2(){
  if(draw_flags.sphr_complex != BEST_SPH){
    change_sphere_complexity(renWindow, 2);
    draw_flags.sphr_complex = 2;
  }

  // Update the render window to reflect the changes
  renWindow->Render();
}

// Set the bond complexity
void set_bond_cmplx0(){
  if(draw_flags.bond_complex != LINE_BOND){
    //change_bond_complexity(ren, renWindow, LINE_BOND);
    delete_bonds(ren);
    draw_flags.bond_complex = LINE_BOND;
    draw_bonds(ren, &draw_flags, frame_curr, element_head);
  }

  // Update the render window to reflect the changes
  renWindow->Render();
}

// Set the bond complexity
void set_bond_cmplx1(){
  if(draw_flags.bond_complex != STRC_BOND){
    //change_bond_complexity(ren, renWindow, STRC_BOND);
    delete_bonds(ren);
    draw_flags.bond_complex = STRC_BOND;
    draw_bonds(ren, &draw_flags, frame_curr, element_head);
  }
  
  // Update the render window to reflect the changes
  renWindow->Render();
}

// Set the bond complexity
void set_bond_cmplx2(){
  if(draw_flags.bond_complex != TAPC_BOND){
    //change_bond_complexity(ren, renWindow, TAPC_BOND);
    delete_bonds(ren);
    draw_flags.bond_complex = TAPC_BOND;
    draw_bonds(ren, &draw_flags, frame_curr, element_head);
  }

  // Update the render window to reflect the changes
  renWindow->Render();
}

// Toggle bonds on/off
void toggle_bonds(){

  if(draw_flags.draw_bonds){
    delete_bonds(ren);
    draw_flags.draw_bonds = false;
    renWindow->Render();
  }
  else{
    draw_bonds(ren, &draw_flags, frame_curr, element_head);
    draw_flags.draw_bonds = true;
    renWindow->Render();
  }
    
}

// Toggle coordinate axes on/off
void toggle_axes(){
  
  if(draw_flags.show_axis){
    delete_axes(ren);
    draw_flags.show_axis = false;
    renWindow->Render();
  }
  else{
    draw_axes(ren);
    draw_flags.show_axis = true;
    renWindow->Render();
  }

}

// Toggle spheres(atoms) on/off
void toggle_atoms(){

  if(draw_flags.draw_atoms){
    delete_atoms(ren);
    draw_flags.draw_atoms = false;
    renWindow->Render();
  }
  else{
    draw_atoms(ren, &draw_flags, frame_curr);
    draw_flags.draw_atoms = true;
    renWindow->Render();
  }
}

// Toggle atom labels on/off
void toggle_alabels(){

  if(draw_flags.show_chr_labs){
    delete_atom_labels(ren);
    draw_flags.show_chr_labs = false;
    renWindow->Render();
  }
  else{
    draw_atom_labels(ren, &draw_flags, frame_curr->atoms);
    draw_flags.show_chr_labs = true;
    renWindow->Render();
  }
}

// Toggle between perspective and orthographic(parallel) projection
void toggle_projection(){

  if(draw_flags.ortho_proj){
    draw_flags.ortho_proj = false;
    ren->GetActiveCamera()->SetParallelProjection(0);    
  }
  else{
    draw_flags.ortho_proj = true;
    ren->GetActiveCamera()->SetParallelProjection(1);
  }

  renWindow->Render();
}

// Render the isosurfaces based on the values specified in the gui
void render_isosurfaces(){

  // If a density file has not been loaded then don't render
  if(isoExtractor == NULL) return;

  // Delete any existing isosurfaces
  delete_surfaces(ren);
  
  draw_flags.draw_surfaces = true;
  set_minmax_values();
  set_contour_values();
  set_opacity_value();
  init_contour_filter();
  draw_isosurfaces(ren);
  renWindow->Render();

}

// Hide the isosurfaces
void hide_isosurfaces(){

  if(draw_flags.draw_surfaces = true){
    delete_surfaces(ren);
    draw_flags.draw_surfaces = false;
    renWindow->Render();
  }

}

GtkWidget* make_button_with_pixmap(char *pic, GtkSignalFunc func){
  GtkWidget *button;
  GtkWidget *pixmap_widget;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  GtkStyle *style;

  char tmp[256];
  char *name;
    
  g_snprintf( tmp, 255, "%s%s.xpm", pixmapdir, pic);
  name = (char *) calloc(strlen(tmp) + 1, 1);
  name = strncpy(name, tmp, strlen(tmp));
  
  style = gtk_widget_get_style(control_window);
  pixmap = gdk_pixmap_create_from_xpm(control_window->window, &mask, &style->bg[GTK_STATE_NORMAL], 
				      (gchar*)name);
  pixmap_widget = gtk_pixmap_new(pixmap, mask);

  button = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(button), pixmap_widget);
  gtk_widget_show(pixmap_widget);

  if(func)
    gtk_signal_connect(GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC(func), NULL);  
    
  gtk_widget_show(button);

  free(name);
  return button;
}	                        

GtkWidget* create_player_buttons(){
  GtkWidget *table;
  GtkWidget *label;
  GtkWidget *gotobutton;
  GtkWidget *b1, *b2, *b3;
  GtkWidget *rw, *ff;
  char frame_num[4];
  
  table = gtk_table_new(TRUE, 3, 4);

  /* TOP ROW */ 
  b1 = make_button_with_pixmap("play_back", play_back);
  b2 = make_button_with_pixmap( "stop", stop);
  b3 = make_button_with_pixmap( "play_forward", play_forward);
  
  gtk_table_attach_defaults(GTK_TABLE(table), b1, 0, 1, 0, 1);
  gtk_table_attach_defaults(GTK_TABLE(table), b2, 1, 2, 0, 1);
  gtk_table_attach_defaults(GTK_TABLE(table), b3, 2, 3, 0, 1);
  
  /* MIDDLE ROW */
  rw = make_button_with_pixmap("rw", prev_frame);
  ff = make_button_with_pixmap("ff", next_frame);
  
  /* Create goto button  */
  gotobutton = make_button_with_pixmap( "goto", NULL);
  
  gtk_table_attach_defaults(GTK_TABLE(table), rw, 0, 1, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), ff, 1, 2, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), gotobutton, 2, 3, 1, 2);
  
  /* Create frame counter */
  frame_counter = gtk_label_new("1");
  gtk_widget_show(frame_counter);
  gtk_table_attach_defaults(GTK_TABLE(table), frame_counter, 0, 1, 2, 3);
  

  label = gtk_label_new("of");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 2, 3);

  /* Create frame count */
  g_snprintf(frame_num, 4, "%d", frame_tail->info->sequence);
  label = gtk_label_new(frame_num);
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 2, 3);

  return table;
  
}

void play_back(){
  char frame_num[4];

  // Clear the picked atoms array
  clearPickedAtomArray(ren, frame_curr, pickedAtoms);

  while(frame_curr->prev != NULL){
    frame_curr = frame_curr->prev;
    draw_frame(ren, renWindow, &draw_flags, frame_curr, element_head);
    g_snprintf(frame_num, 4, "%d", frame_curr->info->sequence);
    gtk_label_set_text(GTK_LABEL(frame_counter), frame_num);
    while (gtk_events_pending()) gtk_main_iteration();
  }

}

void play_forward(){
  char frame_num[4];

  // Clear the picked atoms array
  clearPickedAtomArray(ren, frame_curr, pickedAtoms);

  while(frame_curr->next != NULL){
    frame_curr = frame_curr->next;
    draw_frame(ren, renWindow, &draw_flags, frame_curr, element_head);
    g_snprintf(frame_num, 4, "%d", frame_curr->info->sequence);
    gtk_label_set_text(GTK_LABEL(frame_counter), frame_num);
    while (gtk_events_pending()) gtk_main_iteration();
  }
  
}

void stop(){


}

void next_frame(){
  char frame_num[4];

  if(frame_curr->next != NULL){
    // Clear the picked atoms array
    clearPickedAtomArray(ren, frame_curr, pickedAtoms);

    frame_curr = frame_curr->next;
    draw_frame(ren, renWindow, &draw_flags, frame_curr, element_head);
    g_snprintf(frame_num, 4, "%d", frame_curr->info->sequence);
    gtk_label_set_text(GTK_LABEL(frame_counter), frame_num);
  }

}

void prev_frame(){
  char frame_num[4];

  if(frame_curr != frame_head){
    // Clear the picked atoms array
    clearPickedAtomArray(ren, frame_curr, pickedAtoms);

    frame_curr = frame_curr->prev;
    draw_frame(ren, renWindow, &draw_flags, frame_curr, element_head);
    g_snprintf(frame_num, 4, "%d", frame_curr->info->sequence);
    gtk_label_set_text(GTK_LABEL(frame_counter), frame_num);
  }
}


void create_density_file_window(){
  GtkWidget *filew;

  filew = gtk_file_selection_new("Open File");
  gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button), 
		     "clicked", (GtkSignalFunc) open_ok_density, filew);
  gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION(filew)->cancel_button),
			    "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));
  gtk_widget_show(filew);

}

void open_ok_density(GtkWidget *w, GtkFileSelection *fs){
  char minString[12], maxString[12], opacityString[5];

  // Open the electron density file
  open_density_file(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));

  // Set the min/max entry widgets to the values queried from the dataset
  snprintf(minString, 12, "%f", surface_info.min_density);
  snprintf(maxString, 12, "%f", surface_info.max_density);
  snprintf(opacityString, 5, "%f", surface_info.opacity);
  gtk_entry_set_text(GTK_ENTRY(minEntry), minString);
  gtk_entry_set_text(GTK_ENTRY(maxEntry), maxString);
  gtk_entry_set_text(GTK_ENTRY(opacityEntry), opacityString);

  gtk_widget_destroy(GTK_WIDGET(fs));
}

void create_writePPM_file_window(){
  GtkWidget *filew;

  filew = gtk_file_selection_new("Write PPM");
  gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button), 
		     "clicked", (GtkSignalFunc) open_ok_writePPM, filew);
  gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION(filew)->cancel_button),
			    "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));

  // Show the file selection dialog
  gtk_widget_show(filew);
}


void open_ok_writePPM(GtkWidget *w, GtkFileSelection *fs){
  
  vtkWindowToImageFilter *windowToImage = vtkWindowToImageFilter::New();
  windowToImage->SetInput(renWindow);

  vtkPNMWriter *ppmWriter = vtkPNMWriter::New();
  ppmWriter->SetInput(windowToImage->GetOutput());
  ppmWriter->SetFileName(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
  ppmWriter->Write();

  windowToImage->Delete();
  ppmWriter->Delete();

  // Destroy the file selection dialog
  gtk_widget_destroy(GTK_WIDGET(fs));
}

void create_writeBMP_file_window(){
  GtkWidget *filew;

  filew = gtk_file_selection_new("Write BMP");
  gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button), 
		     "clicked", (GtkSignalFunc) open_ok_writeBMP, filew);
  gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION(filew)->cancel_button),
			    "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));

  // Show the file selection dialog
  gtk_widget_show(filew);
}

void open_ok_writeBMP(GtkWidget *w, GtkFileSelection *fs){
  vtkWindowToImageFilter *windowToImage = vtkWindowToImageFilter::New();
  windowToImage->SetInput(renWindow);

  vtkBMPWriter *bmpWriter = vtkBMPWriter::New();
  bmpWriter->SetInput(windowToImage->GetOutput());
  bmpWriter->SetFileName(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
  bmpWriter->Write();

  windowToImage->Delete();
  bmpWriter->Delete();

  // Destroy the file selection dialog
  gtk_widget_destroy(GTK_WIDGET(fs));
}

void create_writePS_file_window(){
  GtkWidget *filew;

  filew = gtk_file_selection_new("Write Postscript");
  gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button), 
		     "clicked", (GtkSignalFunc) open_ok_writePS, filew);
  gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION(filew)->cancel_button),
			    "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));

  // Show the file selection dialog
  gtk_widget_show(filew);

}

void open_ok_writePS(GtkWidget *w, GtkFileSelection *fs){
  vtkWindowToImageFilter *windowToImage = vtkWindowToImageFilter::New();
  windowToImage->SetInput(renWindow);

  vtkPostScriptWriter *psWriter = vtkPostScriptWriter::New();
  psWriter->SetInput(windowToImage->GetOutput());
  psWriter->SetFileName(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
  psWriter->Write();

  windowToImage->Delete();
  psWriter->Delete();

  // Destroy the file selection dialog
  gtk_widget_destroy(GTK_WIDGET(fs));


}

void create_saveCoord_file_window(){
  GtkWidget *filew;

  filew = gtk_file_selection_new("Open File");
  gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button), 
		     "clicked", (GtkSignalFunc) save_coord_ok, filew);
  gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION(filew)->cancel_button),
			    "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));
  gtk_widget_show(filew);

}

void save_coord_ok(GtkWidget *w, GtkFileSelection *fs){
  FILE *outfile;
  

  if(outfile = fopen(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)), "w+")){
    write_rmf(frame_curr, outfile);
  }

  // Destroy the file selection dialog
  gtk_widget_destroy(GTK_WIDGET(fs));
}

void createReportDialog(char *reportString){
  GtkWidget *window, *button, *label;


  window = gtk_dialog_new();

  // Create the label
  label = gtk_label_new(reportString);
  gtk_box_pack_start(GTK_BOX (GTK_DIALOG (window)->vbox), label, TRUE, TRUE, 0);
  gtk_widget_show(label);

  // Create the close button
  button = gtk_button_new_with_label("Close");
  gtk_box_pack_start(GTK_BOX (GTK_DIALOG (window)->vbox), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT (button), "clicked", (GtkSignalFunc) gtk_widget_destroy,
		     GTK_OBJECT (window));
  gtk_widget_show (button);

  gtk_widget_show(window);
}

