#include <stdio.h>

#include "vtkRenderer.h"
#include "vtkRenderWindow.h"
#include "vtkPolyDataMapper.h"
#include "vtkActor.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkXRenderWindowInteractor.h"
#include "vtkInteractorStyleTrackball.h"
#include "vtkSphereSource.h"
#include "vtkCylinderSource.h"
#include "vtkContourFilter.h"
#include "vtkMarchingCubes.h"
#include "vtkCellPicker.h"

#include <gtk/gtk.h>

#include "defs.h"
#include "types.h"
#include "protos.h"
#include "gtk_protos.h"

#include "local_inits.h"




FRAME_LIST *frame_head = NULL,          // Various frame pointers
           *frame_curr = NULL,
           *frame_tail = NULL;

ELEMENT_LIST *element_head = NULL;      // Head of the element list
   
SURFACE_INFO surface_info;              // Surface info structure

ATOM_LIST *pickedAtoms[3] = { NULL, NULL, NULL} ; // Currently selected atoms

int total_frames = 1;                   // The number of frames 
int viewWidth = 600, viewHeight = 600;  // Initial size of the render window
int num_elements = 0;                   // Number of elements in the table file

// Global vtk objects
vtkSphereSource **spheres[3];
vtkPolyDataMapper *axesMapper;
vtkPolyDataMapper **sphereMappers;
vtkPolyDataMapper *bondMappers[2];
vtkPolyDataMapper *lineMapper;
vtkPolyDataMapper **labelMappers;
vtkPolyDataMapper *surfaceMapper= NULL;
//vtkContourFilter  *isoExtractor = NULL;
vtkMarchingCubes *isoExtractor = NULL;
vtkRenderer *ren;
vtkRenderWindow *renWindow;
vtkXRenderWindowInteractor *iren;


void break_vtk_loop(void *vtkNotUsed(arg));
static void pickObject(void *arg);

int main(int argc, char *argv[]){
  
  // print version info
  fprintf(stderr,"%s -- Version %s\n",PROGNAME,VERSION);

  // parse the command line, get file names to open
  parse_command_line( argc, argv, &file_info, &draw_flags );

  // initialize element list
  init_element_list(&element_head, draw_flags);

  // open and read the initial element table file
  get_first_table(&file_info, element_head);

  // count the number of elements
  num_elements = count_elements(element_head);

  // open and read the initial coordinate file
  get_first_model(&file_info, element_head, &frame_head, 
                  &frame_tail, &draw_flags);
  frame_curr = frame_head;

  // find the scale of the model before we start the graphics
  get_max_radius( frame_head, &draw_flags );


  // Now on to the graphics...
  
  // Create rendering window and renderer
  ren = vtkRenderer::New();
  renWindow = vtkRenderWindow::New();
    renWindow->AddRenderer(ren);
    renWindow->SetSize(viewWidth, viewHeight);
  vtkInteractorStyleTrackball *trackball = vtkInteractorStyleTrackball::New();
    trackball->SetTrackballModeToTrackball();
  iren = vtkXRenderWindowInteractor::New();
    iren->SetInteractorStyle(trackball);
    iren->SetRenderWindow(renWindow);
    iren->SetUserMethod(break_vtk_loop, 0);

  // Setup the interactor's picker
  vtkCellPicker *picker = vtkCellPicker::New();
    picker->SetTolerance(0.005);
    iren->SetPicker(picker);
    picker->Delete();
  iren->SetEndPickMethod(pickObject, (void *)iren);  

  // Set the material colors
  color_setup(element_head, &draw_flags);
  ren->SetBackground(draw_flags.back_color.red,draw_flags.back_color.green,
		     draw_flags.back_color.blue);

  // Create and initialize the graphics objects (vtk actors) for the
  // initial frame.
  init_mappers(&draw_flags, element_head);
  init_actors(ren, &draw_flags, frame_head, element_head);

  // Initialize GTK and create the interface window
  gtk_init(&argc, &argv);
  create_control_window();
  
  // Render the scene in the VTK window
  renWindow->Render();

  // Start the GTK event loop
  gtk_main();
  
}

// This is a user-defined function for the vtkWindowInteractor that
// stops the VTK event loop.  It is called by pressing "u".
// 
void break_vtk_loop(void *vtkNotUsed(arg)){

  if(pickedAtoms[0] != NULL && pickedAtoms[1] != NULL && pickedAtoms[2] != NULL){
    reportAngle(pickedAtoms[0]->x, pickedAtoms[0]->y, pickedAtoms[0]->z,
		pickedAtoms[1]->x, pickedAtoms[1]->y, pickedAtoms[1]->z,
		pickedAtoms[2]->x, pickedAtoms[2]->y, pickedAtoms[2]->z);
    clearPickedAtomArray(ren, frame_curr, pickedAtoms);
    renWindow->Render();
  }
  else if(pickedAtoms[0] != NULL && pickedAtoms[1] != NULL){
    reportDistance(pickedAtoms[0]->x, pickedAtoms[0]->y, pickedAtoms[0]->z,
		   pickedAtoms[1]->x, pickedAtoms[1]->y, pickedAtoms[1]->z);
    clearPickedAtomArray(ren, frame_curr, pickedAtoms);
    renWindow->Render();
  }

}

static void pickObject(void *arg){
  vtkCellPicker *picker = (vtkCellPicker *) iren->GetPicker();
  vtkPolyDataMapper *pickedMapper = (vtkPolyDataMapper *) picker->GetMapper();

  // If it didn't pick an actor then return
  if(pickedMapper == NULL) return;

  if(pickedAtoms[0] == NULL){
    
    pickedAtoms[0] = getPickedAtom(frame_curr, picker, pickedMapper);
    renWindow->Render();

  }
  else if(pickedAtoms[1] == NULL){

    pickedAtoms[1] = getPickedAtom(frame_curr, picker, pickedMapper);
    renWindow->Render();

  }
  else if(pickedAtoms[2] == NULL){
    
    pickedAtoms[2] = getPickedAtom(frame_curr, picker, pickedMapper);
    renWindow->Render();

  }
  else{
    
    clearPickedAtomArray(ren, frame_curr, pickedAtoms);
    pickedAtoms[0] = getPickedAtom(frame_curr, picker, pickedMapper);
    renWindow->Render();
    
  }
  
}
