#include <stdio.h>

#include "vtkRenderer.h"
#include "vtkCellPicker.h"
#include "vtkActorCollection.h"
#include "vtkProp3D.h"
#include "vtkPolyDataMapper.h"
#include "vtkActor.h"

#include "defs.h"
#include "types.h"
#include "protos.h"
#include "gtk_protos.h"

extern int num_elements;
extern vtkPolyDataMapper **sphereMappers;

ATOM_LIST *getPickedAtom(FRAME_LIST *frame, vtkCellPicker *picker, vtkPolyDataMapper *pickedMapper){
  ATOM_LIST *atomCursor, *pickedAtom;
  int i, isAtom;
  float pos[3], pos2[3];
  
  pickedAtom = NULL;
  vtkProp3D *pickedProp3D = picker->GetProp3D();

  // Test to see if the picked actor is an atom
  isAtom = 0;
  i = 0;
  while(!isAtom && i < num_elements){
    if(pickedMapper == sphereMappers[i]) isAtom = 1;
    i++;
  }
  if(isAtom){
    // If we've selected an atom, then we want to return it
    
    // find the matching atom
    pickedProp3D->GetPosition(pos);
    atomCursor = frame->atoms;
    while(atomCursor != NULL && 
	  (pos[0] != atomCursor->x || pos[1] != atomCursor->y || pos[2] != atomCursor->z) ){
      //fprintf(stderr, "%f - %f - %f\n", atomCursor->x, atomCursor->y, atomCursor->z);
      atomCursor = atomCursor->next;
      
    }
    
    // if there is a match, add the atom to the picked atoms list
    if(atomCursor == NULL){
      fprintf(stderr, "Selected atom doesn't match any known atoms!\n");
      return NULL;
    }
    else{
      pickedAtom = atomCursor;
    }
    
    // find the picked atom's actor, and change its color
    vtkActorCollection *pickedActors = picker->GetActors();
    pickedActors->InitTraversal();
    int numPicked = pickedActors->GetNumberOfItems();
    vtkActor *current = pickedActors->GetNextActor();
    for(i = 0; i < numPicked; i++){
      current->GetPosition(pos2);
      if(pos[0] == pos2[0] && pos[1] == pos2[1] && pos[2] == pos2[2]){
	current->GetProperty()->SetColor(0.5, 0.5, 0.5);
	break;
      }
      current = pickedActors->GetNextActor();
    }
    
  }
  
  return pickedAtom;
}

void clearPickedAtomArray(vtkRenderer *ren, FRAME_LIST *frame, ATOM_LIST *pickedAtoms[]){
  int i;
  float pos[3];
  vtkActor *current;
  vtkActorCollection *actors = ren->GetActors();

  // Change the picked atoms back to their original colors
  i = 0;
  while(pickedAtoms[i] != NULL && i < 3){
    actors->InitTraversal();
    current = actors->GetNextActor();
    current->GetPosition(pos);
    while(current != NULL && 
	  (pickedAtoms[i]->x != pos[0] || pickedAtoms[i]->y != pos[1]
	   || pickedAtoms[i]->z != pos[2]) ){
      
      current = actors->GetNextActor();
      current->GetPosition(pos);
    }
    if(current != NULL){
      current->GetProperty()->SetColor(pickedAtoms[i]->info->color.red,
				       pickedAtoms[i]->info->color.green,
				       pickedAtoms[i]->info->color.blue);
    }
    
    i++;
  }

  pickedAtoms[0] = pickedAtoms[1] = pickedAtoms[2] = NULL;
}

void reportAngle(float x1, float y1, float z1, float x2, float y2, float z2,
		 float x3, float y3, float z3){
  char reportString[50];
  
  sprintf(reportString, "The angle is %f\n", angle(x1, y1, z1, x2, y2, z2, x3, y3, z3));
  printf("%s", reportString);
  //createReportDialog(reportString);
}

void reportDistance(float x1, float y1, float z1, float x2,float  y2, float z2){
  char reportString[50];
  
  sprintf(reportString, "The distance is %f\n", distance(x1, y1, z1, x2, y2, z2));
  printf("%s", reportString);
  //createReportDialog(reportString);  
}
