/*
 * Rotmole / Rotmovie source code
 *
 * $RCSfile: rmf_reader.c,v $
 *
 * $Log: rmf_reader.c,v $
 * Revision 3.0  1994/11/21 14:31:21  brabec
 * *** empty log message ***
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "defs.h"
#include "types.h"
#include "protos.h"

/*
 * This routine generates the frame list from coordinates 
 * read in from the opened file.
 *
 * Input: file, element list, frame list, flags
 * Output: new frame list
 */

/* length of lines to read */
#define LINLEN 150

/* Used to calculate bounding box */
#define BB_MAX(x,y) x = (y > x ? y : x)
#define BB_MIN(x,y) x = (y < x ? y : x)

extern int total_frames;

void rmf_reader( FILE *ifil, ELEMENT_LIST *element_head, 
                 FRAME_LIST **frame_head, FRAME_LIST **frame_tail, 
                 DRAW_FLAGS *flags )
{

   TOGGLE done = false;

   float rx,ry,rz;

   char label[LABEL_LEN+1];

   char lin[LINLEN], *index, *scanend;

   int pos, atom_seq=1;

   FRAME_LIST *cur_frame;

   FRAME_INFO *cur_info;

   ATOM_LIST *cur_atom = NULL;

   int lines_parsed = 0,
       report_on;

#ifdef MOVIE

   TOGGLE skipthis = false;

   int skip_count = 0;

   int framenum=0, frames_saved=0;

#endif


/* First, allocate a new frame list */

   cur_frame = alloc_frame_list();
   Check_ptr( cur_frame );
   *frame_head = cur_frame;
   *frame_tail = cur_frame;
   cur_info = cur_frame->info;

/* set up counters for rotmovie */

#ifdef MOVIE

   if (flags->frame_skip != 0) skipthis = true;

#endif

/* set up progress indicator */

#ifdef MOVIE
   fprintf(stderr,"Read: %8d atoms, %6d frames",0,0);
#else
   fprintf(stderr,"Read: %8d atoms",0);
#endif

/* report progress every so many atoms */

   if (flags->calc_bonds) 
     report_on = 2000;
   else
     report_on = 4000;


   while (!done) {

     fgets( lin, LINLEN, ifil );

/* commas to whitespace */
     for (index = lin; *index != 0; ++index)
       if (*index == ',') *index = ' ';

/* jump to first non-space character */
     for (index = lin; isspace((int) *index) && *index != '\n'; ++index)
       ; 

     if (feof(ifil))
       done = true;

     else if (index[0] == '/') 
       {

#ifndef MOVIE

         done = true;

#else

         if (toupper(index[1]) == 'Q')

           done = true;

         else { /* end of frame tag */

           framenum++;
           if (skipthis) { /* when we just skipped a frame */

             skip_count++;

             if (framenum > flags->frame_skip) { 

               if (skip_count == flags->frame_stride-1) skipthis = false;

             } else if (framenum == flags->frame_skip) {

               skipthis = false;

             }

           } else { /* we just read a frame */

             cur_info->sequence = framenum;

             if (flags->calc_bonds) {
               create_bond_table( cur_frame );
             }

             frames_saved++;

             if (frames_saved == flags->max_frames) 
               done = true;

             else { /* start new frame pointer */

               cur_frame->next = alloc_frame_list();
               Check_ptr(cur_frame->next);

               cur_frame->next->prev = cur_frame;
               cur_frame = cur_frame->next;
               cur_atom = NULL;

               *frame_tail = cur_frame;
               cur_info = cur_frame->info;

               /* set skip flags, if needed */

               if (flags->frame_stride > 1) {

                 skipthis = true;
                 skip_count=0;

               }

             } /* new frame ptr */

           } /* just read a frame */

         } /* end of frame tag */

#endif

       }
     else if ( index[0] == '#' )
       ; /* ignore comment line */

#ifdef MOVIE

     else if (skipthis)
       ; /* skipping this frame */

#endif

     else {   /* parse the item line */

/* first, get the atom label */
       for (pos=0; !isspace((int) index[pos]); pos++)
         ; /* scan to next whitespace */

       pos = (pos < LABEL_LEN ? pos : LABEL_LEN);

       strncpy(label, index, pos);
       label[pos] = 0;

/* Next, grab the numbers: x,y,z */
       for ( ; !isspace((int) *index) && *index != '\n'; ++index)
         ; /* scan to next whitespace */

       rx = strtod(index, &scanend);

       index = scanend;
       ry = strtod(index, &scanend);

       index = scanend;
       rz = strtod(index, &scanend);

       if ((rz == 0) && (scanend == index)) {

         fprintf(stderr,"Error parsing line (ignored):\n%s\n",lin);

       } else {

/* Add atom to the current list */

    /* allocate the new atom item */
         if (cur_frame->atoms == NULL) {
            cur_atom = alloc_atom_list();
            Check_ptr( cur_atom );
            cur_frame->atoms = cur_atom;
            atom_seq = 1;
         } else {
            cur_atom->next = alloc_atom_list();
            Check_ptr( cur_atom->next );
            cur_atom = cur_atom->next;
         }

    /* fill the data structure */

         cur_atom->x = rx;
         cur_atom->y = ry;
         cur_atom->z = rz;
         cur_atom->sequence = atom_seq;
         cur_atom->info = find_element(element_head, label);
         atom_seq++;

    /* update the frame bounding box */

         BB_MAX(cur_frame->info->bb_xmax, rx);
         BB_MIN(cur_frame->info->bb_xmin, rx);
         BB_MAX(cur_frame->info->bb_ymax, ry);
         BB_MIN(cur_frame->info->bb_ymin, ry);
         BB_MAX(cur_frame->info->bb_zmax, rz);
         BB_MIN(cur_frame->info->bb_zmin, rz);

    /* update and display line counters */

         lines_parsed++;
         if ((lines_parsed % report_on) == 0) {

#ifdef MOVIE
           fprintf(stderr,
              "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
           fprintf(stderr,"%8d atoms, %6d frames",
                   lines_parsed, frames_saved);
#else
           fprintf(stderr,"\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
           fprintf(stderr,"%8d atoms",
                   lines_parsed);
#endif

         }

       } /* else */

     } /* parse */

   } /* while not done */

/* finish up last frame */

   if (cur_frame->atoms != NULL) {

#ifdef MOVIE

     if (frames_saved != flags->max_frames) {
       frames_saved++;
       framenum++;
       cur_frame->info->sequence = framenum;

       if (flags->calc_bonds) {
         create_bond_table( cur_frame );
       }
     }

#else

     if (flags->calc_bonds)
       create_bond_table( cur_frame );

#endif

   } else {

      if (cur_frame->prev == NULL) {

        fprintf(stderr,"ERROR: No data was read.\n");
        exit(-1);

      } else {

        *frame_tail = cur_frame->prev;
        (*frame_tail)->next = NULL;
        free( (void *) cur_frame->info );
        free( (void *) cur_frame );

      }

   }

/* print a status line */

#ifdef MOVIE
   fprintf(stderr,
      "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
   fprintf(stderr,"%8d atoms, %6d frames\n", lines_parsed, frames_saved);
   total_frames = frames_saved;
#else
   fprintf(stderr,"\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
   fprintf(stderr,"%8d atoms\n", lines_parsed);
#endif


}


/* end of rmf_reader.cxx */
