/*
 * Rotmole / Rotmovie source code
 *
 * $RCSfile: types.h,v $
 *
 * $Log: types.h,v $
 * Revision 3.0  1994/11/21 14:31:21  brabec
 * *** empty log message ***
 *
 */



/* Main Data Structures */

typedef enum {up = 0, down = 1} POS;
typedef enum {black = 0, green = 1, red = 2} BUTTONCOLOR;

typedef bool TOGGLE;

typedef struct {
    float red,
          green,
          blue;
    long  pack;
  }  COLOR;

struct ELEMENT_TYPE {  
    char *label;		/* The element tag */
    float radius;		/* Bonding Radius  */
    TOGGLE can_bond;		/* allowed to bond */
    COLOR color;		/* drawing color   */
    COLOR label_color;		/* labelling color (currently unused) */
    int material;		/* material number */
    int sphere_index;           /* index into the source sphere array */
                                /* also used to index label mappers array */
    struct ELEMENT_TYPE *next;	/* next in list	   */
  } ;

typedef struct ELEMENT_TYPE ELEMENT_LIST;

struct ATOM_TYPE {
    int sequence;		/* Atom label number   */
    float x,y,z;		/* Position            */
    ELEMENT_LIST *info;		/* Ptr to element info */
    struct ATOM_TYPE *next;	/* next in list        */
  } ;

typedef struct ATOM_TYPE ATOM_LIST;

struct BOND_TYPE {
    ATOM_LIST *end1, *end2;	/* ptrs to ends of bond */
    struct BOND_TYPE *next;	/* next in list */
  } ;

typedef struct BOND_TYPE BOND_LIST;

typedef struct {
    int sequence;		/* frame number      */
    float bb_xmin,		/* bounding box in X */
          bb_xmax;
    float bb_ymin,		/* bounding box in Y */
          bb_ymax;
    float bb_zmin,		/* bounding box in Z */
          bb_zmax;
  } FRAME_INFO;

struct FRAME_TYPE {
    FRAME_INFO *info;			/* frame information */
    ATOM_LIST *atoms;			/* list of atoms     */
    BOND_LIST *bonds;			/* list of bonds     */
    struct FRAME_TYPE *prev, *next;	/* prev, next in list  */
  } ;

typedef struct FRAME_TYPE FRAME_LIST;

typedef struct {
  int   num_surfaces;
  double min_density, max_density;
  double contour_values[6];
  float opacity;
  float specular;
  int   spec_power;
   
} SURFACE_INFO;


/* Control Flags */



typedef struct {
    TOGGLE show_axis,		/* show axis flag             */
           show_seq_labs,	/* show sequence labels       */
           show_chr_labs,	/* show character labels      */
           draw_bonds,		/* draw bonds flag            */
           draw_atoms,          /* draw atoms flag            */      
           draw_surfaces,       /* draw density iso surfaces  */
           ortho_proj,		/* orthogonal projection flag */
#ifdef MOVIE
           frame_ctr,		/* show frame counter flag    */
#endif
           video_screen,	/* video output flag          */
           calc_bonds;		/* calculate bonds on startup */

    int sphr_complex,		/* sphere complexity          */
        bond_complex;		/* bond complexity            */

#ifdef MOVIE
    int frame_skip,		/* frames to skip during read */
        frame_stride,		/* frames to stride "     "   */
        max_frames;		/* maximum frames to read     */
#endif

    float zoom_level,		/* current magnification      */
          sphr_size,		/* sphere drawing size        */
          bond_size,		/* bond drawing size          */
          max_radius,		/* maximum radius if model    */
          avg_radius;		/* average atom radius        */

    COLOR back_color,		/* background color           */
          bond_color,		/* bond color                 */
          axis_color;           /* axis color                 */

#ifdef MOVIE
    COLOR cntr_color;		/* frame counter color        */
#endif

  } DRAW_FLAGS;


#ifdef MOVIE
typedef struct {
    int motion_dir;		/* current direction of play */
    int motion_mode;            /* 0 = one way, 1 = loop, 2 = yoyo */
    POS rframe,		
        rplay,
        stop,
        fplay,
        fframe,
        mode,
        rec,
        slider;
  } VCR_FLAGS;
#endif



/* Menu and Window Structures */



typedef struct {
    int draw,			/* drawing window     */
         vcr_menu,		/* VCR control window */
#ifdef MOVIE
#endif
         current_focus;		/* window in focus    */
  } WINDOW_IDS;

typedef struct {
    long main,			/* main popup id   */
         toggle,		/* toggle id       */
         function,		/* function id     */
         printsave,		/* print/save id   */
         sphr_level,		/* sphere level id */
         sphr_size,		/* sphere size id  */
         bond_level,            /* bond level id   */
         bond_size;             /* bond size id    */
  } MENU_IDS;


#ifdef MOVIE

typedef struct {
    long xmin, xmax,		/* x-dimensions of button */
         ymin, ymax;		/* y-dimensions of button */
    char *label;		/* button label           */
    TOGGLE locked;		/* lock state of button   */
  } BUTTON_INFO;

typedef struct {
    struct {
        long xmin, xmax,	/* prefered x position */
             ymin, ymax;        /* prefered y position */
      } prefs;
    COLOR back_color;		/* background color           */
    long xsize, ysize;          /* size of window      */
    int num_buttons;            /* number of buttons   */
  } VCR_INFO;

#endif



/* File related structure */



typedef struct {
    char *position_fname,	/* position file name */
         *element_fname,	/* element file name  */
         *matrix_fname,		/* matrix file name   */
         *print_fname;		/* print file name    */
 
    int prnt_file_number,	/* print file number  */
        save_file_number;	/* save file number   */
  } FILE_INFO;

/* end of types.h */
