#include <fcntl.h>
#include <stdio.h>
#include <malloc.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

/* Converts a raw dump of electron densities into a .vtk file. */

int main(int argc, char **argv){
   FILE *in, *out;  
   double *data;    /* Buffer to hold the density data */
   int x, y, z;     /* Dimensions of the dataset */
   int datasize;    /* Number of data points in density file */
   int numdigits;
   char sizestring[20];
   
   if(argc != 9){
      fprintf(stderr, "usage: dens2vtk x y z xspacing yspacing zspacing inputfile outputfile\n");
      fprintf(stderr, "  'x', 'y', and 'z' are the dimensions of the dataset\n");
      fprintf(stderr, "  the 'spacing' parameters are the dimensions of each voxel\n");
      return -1;
   }
   
   x = atoi(argv[1]);
   y = atoi(argv[2]);
   z = atoi(argv[3]);
   
   datasize = x * y * z;
   
   /* read in the contents of the density file */
   data = malloc(sizeof(double) * datasize);
   in = fopen(argv[7], "rb");
   fread(data, sizeof(double), datasize, in);
   
   out = fopen(argv[8], "wb");
   fwrite("# vtk DataFile Version 1.0\n", sizeof(char), 27, out);
   fwrite("Electron Density\n", sizeof(char), 17, out);
   fwrite("\nBINARY\n\n", sizeof(char), 9, out);
   fwrite("DATASET STRUCTURED_POINTS\n", sizeof(char), 26, out);
   fwrite("DIMENSIONS ", sizeof(char), 11, out);
   fwrite(argv[1], sizeof(char), strlen(argv[1]), out);
   fwrite(" ", sizeof(char), 1, out);
   fwrite(argv[2], sizeof(char), strlen(argv[2]), out);
   fwrite(" ", sizeof(char), 1, out);
   fwrite(argv[3], sizeof(char), strlen(argv[3]), out);
   fwrite("\n", sizeof(char), 1, out);
   fwrite("SPACING ", sizeof(char), 8, out);
   fwrite(argv[4], sizeof(char), strlen(argv[4]), out);
   fwrite(" ", sizeof(char), 1, out);
   fwrite(argv[5], sizeof(char), strlen(argv[5]), out);
   fwrite(" ", sizeof(char), 1, out);
   fwrite(argv[6], sizeof(char), strlen(argv[6]), out);
   fwrite("\n", sizeof(char), 1, out);
   fwrite("ORIGIN 0.0 0.0 0.0\n\n", sizeof(char), 20, out);
   fwrite("POINT_DATA ", sizeof(char), 11, out);
   
   sprintf(sizestring, "%d", datasize);
   fwrite(sizestring, sizeof(char), strlen(sizestring), out);
   
   fwrite("\n", sizeof(char), 1, out);
   fwrite("SCALARS scalars double\n", sizeof(char), 23, out);
   fwrite("LOOKUP_TABLE default\n", sizeof(char), 21, out); 
   fwrite(data, sizeof(double), datasize, out);
}
